@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2017, 2018 The Sympa Community. See the AUTHORS.md file at the
-# top-level directory of this distribution and at
+# Copyright 2017, 2018, 2019, 2021 The Sympa Community. See the
+# AUTHORS.md file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -108,6 +108,13 @@ sub connect {
             $self);
         return 1;
     }
+    # Disconnected: Transaction (if any) was aborted.
+    if (delete $self->{_sdbTransactionLevel}) {
+        $log->syslog('err', 'Transaction on database %s was aborted: %s',
+            $self, $DBI::errstr);
+        $self->set_persistent($self->{_sdbPrevPersistency});
+        return undef;
+    }
 
     # Do we have required parameters?
     foreach my $param (@{$self->required_parameters}) {
@@ -406,6 +413,56 @@ sub prepare_query_log_values {
 # DEPRECATED: Use tools::eval_in_time() and fetchall_arrayref().
 #sub fetch();
 
+sub begin {
+    my $self = shift;
+
+    my $dbh = $self->__dbh;
+    return undef unless $dbh;
+
+    return undef unless $dbh->begin_work;
+
+    $self->{_sdbTransactionLevel} //= 0;
+    unless ($self->{_sdbTransactionLevel}++) {
+        $self->{_sdbPrevPersistency} = $self->set_persistent(0);
+    }
+
+    return 1;
+}
+
+sub _finalize_transaction {
+    my $self = shift;
+
+    unless (defined $self->{_sdbTransactionLevel}) {
+        return;
+    }
+    unless ($self->{_sdbTransactionLevel}) {
+        die 'bug in logic. Ask developer';
+    }
+    unless (--$self->{_sdbTransactionLevel}) {
+        $self->set_persistent($self->{_sdbPrevPersistency});
+    }
+}
+
+sub commit {
+    my $self = shift;
+
+    my $dbh = $self->__dbh;
+    return undef unless $dbh;
+
+    $self->_finalize_transaction;
+    return $dbh->commit;
+}
+
+sub rollback {
+    my $self = shift;
+
+    my $dbh = $self->__dbh;
+    return undef unless $dbh;
+
+    $self->_finalize_transaction;
+    return $dbh->rollback;
+}
+
 sub disconnect {
     my $self = shift;
 
@@ -434,12 +491,14 @@ sub set_persistent {
     my $self = shift;
     my $flag = shift;
 
+    my $ret = $persistent_connection_of{$self->get_id};
     if ($flag) {
         $persistent_connection_of{$self->get_id} = 1;
     } elsif (defined $flag) {
         delete $persistent_connection_of{$self->get_id};
     }
-    return $self;
+    # Returns the previous value of the flag (6.2.65b.1 or later)
+    return $ret;
 }
 
 sub ping {
@@ -534,6 +593,16 @@ TBD.
 I<Constructor>.
 Creates new database instance.
 
+=item begin ( )
+
+I<Instance method>, I<only for SQL>.
+Begin transaction.
+
+=item commit ( )
+
+I<Instance method>, I<only for SQL>.
+Commit transaction.
+
 =item do_operation ( $operation, options... )
 
 I<Instance method>, I<only for LDAP>.
@@ -564,11 +633,16 @@ Returns:
 
 Statement handle (L<DBI::st> object or such), or C<undef>.
 
+=item rollback ( )
+
+I<Instance method>, I<only for SQL>.
+Rollback transaction.
+
 =back
 
 =head1 SEE ALSO
 
-L<Sympa::DatabaseDriver>, L<Sympa::Datasource>.
+L<Sympa::DatabaseDriver>.
 
 =head1 HISTORY
 
