@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2018 The Sympa Community. See the AUTHORS.md file at the
-# top-level directory of this distribution and at
+# Copyright 2018, 2021 The Sympa Community. See the
+# AUTHORS.md file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -63,6 +63,8 @@ sub connect {
     } else {
         $self->__dbh->func(5000, 'busy_timeout');
     }
+    # Create a temoprarhy view "dual" for portable SQL statements.
+    $self->__dbh->do(q{CREATE TEMPORARY VIEW dual AS SELECT 'X' AS dummy;});
 
     return 1;
 }
@@ -518,10 +520,50 @@ sub translate_type {
     return $type;
 }
 
+# As SQLite does not support nested transactions, these are not effective
+# during when {_sdbSQLiteTransactionLevel} attribute is positive, i.e. only
+# the outermost transaction will be available.
+sub begin {
+    my $self = shift;
+
+    $self->{_sdbSQLiteTransactionLevel} //= 0;
+
+    if ($self->{_sdbSQLiteTransactionLevel}++) {
+        return 1;
+    }
+    return $self->SUPER::begin;
+}
+
+sub commit {
+    my $self = shift;
+
+    unless ($self->{_sdbSQLiteTransactionLevel}) {
+        die 'bug in logic. Ask developer';
+    }
+    if (--$self->{_sdbSQLiteTransactionLevel}) {
+        return 1;
+    }
+    return $self->SUPER::commit;
+}
+
+sub rollback {
+    my $self = shift;
+
+    unless ($self->{_sdbSQLiteTransactionLevel}) {
+        die 'bug in logic. Ask developer';
+    }
+    if (--$self->{_sdbSQLiteTransactionLevel}) {
+        return 1;
+    }
+    return $self->SUPER::rollback;
+}
+
 # Note:
-# To prevent "database is locked" error, acquire "immediate" lock
-# by each query.  Most queries excluding "SELECT" need to lock in this
-# manner.
+# - To prevent "database is locked" error, acquire "immediate" lock
+#   by each query.  Most queries excluding "SELECT" need to lock in this
+#   manner.
+# - If a transaction has been begun, lock is not needed, because SQLite
+#   does not support nested transactions.
 sub do_query {
     my $self = shift;
     my $sth;
@@ -529,8 +571,8 @@ sub do_query {
 
     my $need_lock =
         ($_[0] =~
-            /^\s*(ALTER|CREATE|DELETE|DROP|INSERT|REINDEX|REPLACE|UPDATE)\b/i
-        );
+            /^\s*(ALTER|CREATE|DELETE|DROP|INSERT|REINDEX|REPLACE|UPDATE)\b/i)
+        unless $self->{_sdbSQLiteTransactionLevel};
 
     ## acquire "immediate" lock
     unless (!$need_lock or $self->__dbh->begin_work) {
@@ -569,8 +611,8 @@ sub do_prepared_query {
 
     my $need_lock =
         ($_[0] =~
-            /^\s*(ALTER|CREATE|DELETE|DROP|INSERT|REINDEX|REPLACE|UPDATE)\b/i
-        );
+            /^\s*(ALTER|CREATE|DELETE|DROP|INSERT|REINDEX|REPLACE|UPDATE)\b/i)
+        unless $self->{_sdbSQLiteTransactionLevel};
 
     ## acquire "immediate" lock
     unless (!$need_lock or $self->__dbh->begin_work) {
