@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2017, 2018, 2019, 2020 The Sympa Community. See the AUTHORS.md
-# file at the top-level directory of this distribution and at
+# Copyright 2017, 2018, 2019, 2020, 2021 The Sympa Community. See the
+# AUTHORS.md file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -535,12 +535,12 @@ sub _compile_scenario {
         }
     }
 
-    ## Include a Blacklist rules if configured for this action
-    if ($function and $Conf::Conf{'blacklist'}{$function}) {
+    ## Include a Blocklist rules if configured for this action
+    if ($function and $Conf::Conf{'blocklist'}{$function}) {
         ## Add rules at the beginning of the array
         unshift @rules,
             {
-            'condition'   => "search('blacklist.txt',[sender])",
+            'condition'   => "search('blocklist.txt',[sender])",
             'action'      => 'reject,quiet',
             'auth_method' => ['smtp', 'dkim', 'md5', 'pgp', 'smime'],
             };
@@ -1424,9 +1424,9 @@ sub do_search {
             'order' => 'all'
         );
 
-        ## Raise an error except for blacklist.txt
+        ## Raise an error except for blocklist.txt
         unless (@files) {
-            if ($filter_file eq 'blacklist.txt') {
+            if ($filter_file eq 'blocklist.txt') {
                 return 0;
             } else {
                 $log->syslog('err', 'Could not find search filter %s',
