@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2017, 2018, 2019, 2020 The Sympa Community. See the AUTHORS.md
-# file at the top-level directory of this distribution and at
+# Copyright 2017, 2018, 2019, 2020, 2021 The Sympa Community. See the
+# AUTHORS.md file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -1366,9 +1366,12 @@ sub upgrade {
                     and length $msg_string
                     and $recipient) {
                     $msg_string = MIME::Base64::decode_base64($msg_string);
+                    # Note: See also upgrading from versions later than 6.2b.3
+                    # to version 6.2.63b.1 in below.
+                    # below.
                     my $bounce_path = sprintf '%s/%s_%08s',
                         $list->get_bounce_dir,
-                        Sympa::Tools::Text::escape_chars($recipient),
+                        _escape_chars($recipient),
                         $info->{'pk_notification'};
                     if (open my $fh, '>', $bounce_path) {
                         print $fh $msg_string;
@@ -2070,6 +2073,149 @@ sub upgrade {
         }
     }
 
+    if (lower_version($previous_version, '6.2.61b.1')) {
+        # Variable tags "($tag$% ... %$tag$)" no longer used in
+        # mhonarc_rc.tt2 (ex. mhonarc-ressources.tt2) and were replaced with
+        # "<% ... %>".
+        $log->syslog('notice', 'Converting mhonarc-ressources.tt2...');
+        _process_all_files(
+            'mhonarc-ressources.tt2',
+            sub {
+                my $that    = shift;
+                my $dir     = shift;
+                my $oldfile = shift;
+                my $newfile = 'mhonarc_rc.tt2';
+
+                open my $ifh, '<', $dir . '/' . $oldfile
+                    or die sprintf '%s: %s', $oldfile, $ERRNO;
+                my $text = do { local $RS; <$ifh> };
+                close $ifh;
+
+                $text =~ s{[(]\$tag\$%}{<%}g;
+                $text =~ s{%\$tag\$[)]}{%>}g;
+
+                open my $ofh, '>', $dir . '/' . $newfile
+                    or die sprintf '%s: %s', $newfile, $ERRNO;
+                print $ofh $text;
+                close $ofh;
+            }
+        );
+        $log->syslog('notice', '...Done. Use new file(s) mhonarc_rc.tt2.');
+
+        # blocklist.txt will be used instead of blacklist.txt
+        $log->syslog('notice', 'Rename blacklist.txt to blocklist.txt...');
+        _process_all_files(
+            'search_filters/blacklist.txt',
+            sub {
+                my $that    = shift;
+                my $dir     = shift;
+                my $oldfile = shift;
+                my $newfile = 'search_filters/blocklist.txt';
+
+                rename($dir . '/' . $oldfile, $dir . '/' . $newfile)
+                    or
+                    $log->syslog('err', 'Cannot rename file %s/%s to %s: %m',
+                    $dir, $oldfile, $newfile);
+            }
+        );
+        $log->syslog('notice', '...Done.');
+    }
+
+    if (lower_version($previous_version, '6.2.63b.1')) {
+        $log->syslog('notice', 'Moving bounce information and so on...');
+        _process_all_files(
+            'config',
+            sub {
+                my $that = shift;
+                my $dir  = shift;
+                my $file = shift;
+
+                return unless ref $that eq 'Sympa::List';
+
+                # Note: See also upgrading to version 6.2b.3 in above.
+                my $bounce_dir = $that->get_bounce_dir;
+                my $dh;
+                return unless -d $bounce_dir and opendir $dh, $bounce_dir;
+
+                foreach my $old (readdir $dh) {
+                    next if 0 == index $old, '.';
+
+                    next unless $old =~ /\A(.+)(?:_(\w+))?\z/;
+                    my ($escaped_email, $envid) = ($1, $2);
+
+                    my $new;
+                    if (defined $envid) {
+                        $new = sprintf '%s/%s__%08s',
+                            Sympa::Tools::Text::encode_filesystem_safe(
+                            _unescape_chars($escaped_email)), $envid;
+                    } else {
+                        $new =
+                            Sympa::Tools::Text::encode_filesystem_safe(
+                            _unescape_chars($escaped_email));
+                    }
+
+                    next if $old eq $new;
+
+                    rename sprintf('%s/%s', $bounce_dir, $old),
+                        sprintf('%s/%s', $bounce_dir, $new);
+
+                }
+            }
+        );
+
+        my $dh;
+        if (opendir $dh, $Conf::Conf{'ssl_cert_dir'}) {
+            foreach my $old (readdir $dh) {
+                next if 0 == index $old, '.';
+
+                my ($escaped_email, $ext) = ($old =~ /\A(.+)(?:(\@\w+)?)\z/);
+                my $new =
+                    Sympa::Tools::Text::encode_filesystem_safe(
+                    _unescape_chars($escaped_email))
+                    . ($ext // '');
+                next if $old eq $new;
+
+                rename sprintf('%s/%s', $Conf::Conf{'ssl_cert_dir'}, $old),
+                    sprintf('%s/%s', $Conf::Conf{'ssl_cert_dir'}, $new);
+            }
+        }
+
+        $log->syslog('notice', '...Done.');
+    }
+
+    if (lower_version($previous_version, '6.2.65b.1')) {
+        # Site/domain parameter "tracking" has been deprecated and
+        # should be renamed to "tracking.tracking" to avoid conflict with
+        # list config paragraph named "tracking".
+
+        _process_all_files(
+            '',
+            sub {
+                my $that    = shift;
+                my $dir     = shift;
+                my $oldfile = shift;
+
+                my $file;
+                if (ref $that eq 'Sympa::List') {
+                    return;
+                } elsif ($that and $that ne '*') {
+                    $file = sprintf '%s/robot.conf', $dir;
+                } else {
+                    $file = Sympa::Constants::CONFIG();
+                }
+
+                open my $fh, '<+', $file or next;
+
+                my $text = do { local $RS; <$fh> };
+                $text =~ s/(\A|\n)tracking(\s|\z)/${1}tracking.tracking$2/g;
+                seek $fh, 0, 0;
+                truncate $fh, 0;
+                print $fh $text;
+                close $fh;
+            }
+        );
+    }
+
     return 1;
 }
 
@@ -2383,4 +2529,66 @@ sub _get_cacnonical_write_date {
     }
 }
 
+sub _process_all_files {
+    my $file = shift;
+    my $sub  = shift;
+
+    $sub->('*', $Conf::Conf{'etc'}, $file)
+        if -f $Conf::Conf{'etc'} . '/' . $file;
+
+    foreach my $robot (Sympa::List::get_robots()) {
+        my $dir = sprintf '%s/%s', $Conf::Conf{'etc'}, $robot;
+        $sub->($robot, $dir, $file)
+            if -f $dir . '/' . $file;
+
+        foreach my $list (@{Sympa::List::get_lists($robot) || []}) {
+            $sub->($list, $list->{'dir'}, $file)
+                if -f $list->{'dir'} . '/' . $file;
+        }
+    }
+}
+
+# Old name: tools::escape_chars(), Sympa::Tools::Text::escape_chars().
+sub _escape_chars {
+    my $s          = shift;
+    my $except     = shift;                            ## Exceptions
+    my $ord_except = ord $except if defined $except;
+
+    ## Escape chars
+    ##  !"#$%&'()+,:;<=>?[] AND accented chars
+    ## escape % first
+    foreach my $i (
+        0x25,
+        0x20 .. 0x24,
+        0x26 .. 0x2c,
+        0x3a .. 0x3f,
+        0x5b, 0x5d,
+        0x80 .. 0x9f,
+        0xa0 .. 0xff
+    ) {
+        next if defined $ord_except and $i == $ord_except;
+        my $hex_i = sprintf "%lx", $i;
+        $s =~ s/\x$hex_i/%$hex_i/g;
+    }
+    ## Special traetment for '/'
+    $s =~ s/\//%a5/g unless defined $except and $except eq '/';
+
+    return $s;
+}
+
+# Old name: tools::unescape_chars(), Sympa::Tools::Text::unescape_chars().
+sub _unescape_chars {
+    my $s = shift;
+
+    $s =~ s/%a5/\//g;    ## Special traetment for '/'
+    foreach my $i (0x20 .. 0x2c, 0x3a .. 0x3f, 0x5b, 0x5d, 0x80 .. 0x9f,
+        0xa0 .. 0xff) {
+        my $hex_i = sprintf "%lx", $i;
+        my $hex_s = sprintf "%c",  $i;
+        $s =~ s/%$hex_i/$hex_s/g;
+    }
+
+    return $s;
+}
+
 1;
