@@ -5,6 +5,7 @@
 use strict;
 use warnings;
 use Cwd qw();
+use English;    # FIXME: Avoid $MATCH usage
 use Getopt::Long;
 use Pod::Usage;
 
@@ -29,11 +30,6 @@ my %type_of_entries;
 my %Lexicon;
 ## All the strings, in the order they were found while parsing the files
 my @ordered_strings = ();
-## One occurence of each string, in the order they were found while parsing
-## the files
-my @unique_keys = ();
-## A hash used for control when filling @unique_keys
-my %unique_keys;
 
 ## Retrieving options.
 my %opts;
@@ -53,20 +49,16 @@ if ($opts{version}) {
     exit;
 }
 
-# Initiliazing tags with defaults if necessary.
-# Defaults stored separately because GetOptions append arguments to defaults.
-# Building the string to insert into the regexp that will search strings to
-# extract.
-my $available_tags = join('|', @{$opts{t} || []}) || 'locdt|loc';
-
 if ($opts{'files-from'}) {
     my $ifh;
-    open $ifh, '<', $opts{'files-from'} or die "$opts{'files-from'}: $!\n";
+    open $ifh, '<', $opts{'files-from'}
+        or die sprintf "%s: %s\n", $opts{'files-from'}, $ERRNO;
     my @files = grep { /\S/ and !/\A\s*#/ } split /\r\n|\r|\n/,
-        do { local $/; <$ifh> };
+        do { local $RS; <$ifh> };
     my $cwd = Cwd::getcwd();
-    if ($opts{'directory'}) {
-        chdir $opts{'directory'} or die "$opts{'directory'}: $!\n";
+    if ($opts{directory}) {
+        chdir $opts{directory}
+            or die sprintf "%s: %s\n", $opts{directory}, $ERRNO;
     }
     @ARGV = map { (glob $_) } @files;
     chdir $cwd;
@@ -74,67 +66,21 @@ if ($opts{'files-from'}) {
     @ARGV = ('-');
 }
 
-## Ordering files to present the most interresting strings to translate first.
-my %files_to_parse;
-foreach my $file_to_parse (@ARGV) {
-    $files_to_parse{$file_to_parse} = 1;
-}
-my %favoured_files;
-my @ordered_files;
-my @planned_ordered_files = (
-    "../web_tt2/help.tt2",       "../web_tt2/help_introduction.tt2",
-    "../web_tt2/help_user.tt2",  "../web_tt2/help_admin.tt2",
-    "../web_tt2/home.tt2",       "../web_tt2/login.tt2",
-    "../web_tt2/main.tt2",       "../web_tt2/title.tt2",
-    "../web_tt2/menu.tt2",       "../web_tt2/login_menu.tt2",
-    "../web_tt2/your_lists.tt2", "../web_tt2/footer.tt2",
-    "../web_tt2/list_menu.tt2",  "../web_tt2/list_panel.tt2",
-    "../web_tt2/admin.tt2",      "../web_tt2/list_admin_menu.tt2"
-);
-foreach my $file (@planned_ordered_files) {
-    if ($files_to_parse{$file}) {
-        @ordered_files = (@ordered_files, $file);
-    }
-}
-my @ordered_directories =
-    ("../web_tt2", "../mail_tt2", "../src/etc/scenari", "../src/etc");
-
-foreach my $file (@ordered_files) {
-    $favoured_files{$file} = 1;
-}
-## Sorting by directories
-foreach my $dir (@ordered_directories) {
-    foreach my $file (@ARGV) {
-        unless ($favoured_files{$file}) {
-            if ($file =~ /^$dir/g) {
-                @ordered_files = (@ordered_files, $file);
-                $favoured_files{$file} = 1;
-            }
-        }
-    }
-}
-
-## Sorting by files
-foreach my $file (@ARGV) {
-    unless ($favoured_files{$file}) {
-        @ordered_files = (@ordered_files, $file);
-    }
-}
-
-## Gathering strings in the source files.
-## They will finally be stored into %file
+# Gathering strings in the source files.
+# They will finally be stored into %file.
 
 my $cwd = Cwd::getcwd();
-if ($opts{'directory'}) {
-    chdir $opts{'directory'} or die "$opts{'directory'}: $!\n";
+if ($opts{directory}) {
+    chdir $opts{directory}
+        or die sprintf "%s: %s\n", $opts{directory}, $ERRNO;
 }
 
-foreach my $file (@ordered_files) {
-    next if ($file =~ /\.po.?$/i);    # Don't parse po files
-    my $filename = $file;
-    printf STDOUT "Processing $file...\n";
+foreach my $file (@ARGV) {
+    next if $file =~ m{ [.] po.? \z }ix;    # Don't parse po files
+
+    printf STDOUT "Processing %s...\n", $file;
     unless (-f $file) {
-        print STDERR "Cannot open $file\n";
+        printf STDERR "Cannot open %s\n", $file;
         next;
     }
 
@@ -144,231 +90,20 @@ foreach my $file (@ordered_files) {
         next;
     }
 
-    open my $fh, '<', $file or die "$file: $!\n";
-    $_ = do { local $/; <$fh> };
+    open my $fh, '<', $file or die sprintf "%s: %s\n", $file, $ERRNO;
+    $_ = do { local $RS; <$fh> };
     close $fh;
-    $filename =~ s!^./!!;
-    my $line;
 
-    # Template Toolkit: [%|loc(...)%]...[%END%]
-    $line = 1;
-    pos($_) = 0;
-    while (
-        m!\G.*?\[%[-=~+]?\s*\|\s*($available_tags)(.*?)\s*[-=~+]?%\](.*?)\[%[-=~+]?\s*END\s*[-=~+]?%\]!sg
-    ) {
-        my ($this_tag, $vars, $str) = ($1, $2, $3);
-        $line += (() = ($& =~ /\n/g));    # cryptocontext!
-        $str =~ s/\\\'/\'/g;
-        $vars =~ s/^\s*\(//;
-        $vars =~ s/\)\s*$//;
-        my $expression = {
-            'expression' => $str,
-            'filename'   => $filename,
-            'line'       => $line,
-            'vars'       => $vars
-        };
-        $expression->{'type'} = 'date' if ($this_tag eq 'locdt');
-        &add_expression($expression);
+    if ($file =~ m{ [.] (pm | pl | fcgi) ([.]in)? \z }x) {
+        load_perl($file, $_);
     }
 
-    # Template Toolkit: [% "..." | loc(...) %]
-    $line = 1;
-    pos $_ = 0;
-    while (
-        m{
-        \G .*?
-        \[ % [-=~+]? \s*
-        (?: \' ((?:\\.|[^'\\])*) \' | \" ((?:\\.|[^"\\])*) \" ) \s*
-        \| \s*
-        ($available_tags)
-        (.*?)
-        \s* [-=~+]? % \]
-    }sgx
-    ) {
-        my $str      = $1 || $2;
-        my $this_tag = $3;
-        my $vars     = $4;
-
-        $line += (() = ($& =~ /\n/g));
-        $str =~ s{\\(.)}{
-            ($1 eq 't') ? "\t" :
-            ($1 eq 'n') ? "\n" :
-            ($1 eq 'r') ? "\r" :
-            $1
-        }eg;
-        $vars =~ s/^\s*[(](.*?)[)].*/$1/ or $vars = '';
-
-        my $expression = {
-            'expression' => $str,
-            'filename'   => $filename,
-            'line'       => $line,
-            'vars'       => $vars
-        };
-        $expression->{'type'} = 'date' if ($this_tag eq 'locdt');
-        &add_expression($expression);
+    if ($file =~ m{ [.] tt2 \z }x) {
+        load_tt2($file, $_, $opts{t});
     }
 
-    # Template Toolkit with ($tag$%|loc%$tag$)...($tag$%END%$tag$) in archives
-    $line = 1;
-    pos($_) = 0;
-    while (
-        m!\G.*?\(\$tag\$%\s*\|($available_tags)(.*?)\s*%\$tag\$\)(.*?)\(\$tag\$%[-=~+]?\s*END\s*[-=~+]?%\$tag\$\)!sg
-    ) {
-        my ($this_tag, $vars, $str) = ($1, $2, $3);
-        $line += (() = ($& =~ /\n/g));    # cryptocontext!
-        $str =~ s/\\\'/\'/g;
-        $vars =~ s/^\s*\(//;
-        $vars =~ s/\)\s*$//;
-        my $expression = {
-            'expression' => $str,
-            'filename'   => $filename,
-            'line'       => $line,
-            'vars'       => $vars
-        };
-        $expression->{'type'} = 'date' if ($this_tag eq 'locdt');
-        &add_expression($expression);
-    }
-
-    # Sympa variables (gettext_comment, gettext_id and gettext_unit)
-    $line = 1;
-    pos($_) = 0;
-    while (
-        /\G.*?(\'?)(gettext_comment|gettext_id|gettext_unit)\1\s*=>\s*\"((\\.|[^\"])+)\"/sg
-    ) {
-        my $str = $3;
-        $line += (() = ($& =~ /\n/g));    # cryptocontext!
-        $str =~ s{(\\.)}{eval "\"$1\""}esg;
-        &add_expression(
-            {   'expression' => $str,
-                'filename'   => $filename,
-                'line'       => $line
-            }
-        );
-    }
-
-    $line = 1;
-    pos($_) = 0;
-    while (
-        /\G.*?(\'?)(gettext_comment|gettext_id|gettext_unit)\1\s*=>\s*\'((\\.|[^\'])+)\'/sg
-    ) {
-        my $str = $3;
-        $line += (() = ($& =~ /\n/g));    # cryptocontext!
-        $str =~ s{(\\.)}{eval "'$1'"}esg;
-        &add_expression(
-            {   'expression' => $str,
-                'filename'   => $filename,
-                'line'       => $line
-            }
-        );
-    }
-
-    # Sympa scenarios variables (title.gettext)
-    $line = 1;
-    pos($_) = 0;
-    while (/\G.*?title[.]gettext\s*([^\n]+)/sg) {
-        my $str = $1;
-        $line += (() = ($& =~ /\n/g));    # cryptocontext!
-        &add_expression(
-            {   'expression' => $str,
-                'filename'   => $filename,
-                'line'       => $line
-            }
-        );
-    }
-
-    # Perl source file
-    my ($state, $str, $vars) = (0);
-    my $is_date   = 0;
-    my $is_printf = 0;
-
-    pos($_) = 0;
-    my $orig = 1 + (() = ((my $__ = $_) =~ /\n/g));
-PARSER: {
-        $_ = substr($_, pos($_)) if (pos($_));
-        my $line = $orig - (() = ((my $__ = $_) =~ /\n/g));
-        # maketext or loc or _
-        $state == NUL
-            && m/\b(translate|gettext(?:_strftime|_sprintf)?|maketext|__?|loc|x)/gcx
-            && do {
-            if ($& eq 'gettext_strftime' or $& eq 'gettext_sprintf') {
-                $state     = BEGM;
-                $is_date   = ($& eq 'gettext_strftime');
-                $is_printf = ($& eq 'gettext_sprintf');
-            } else {
-                $state     = BEG;
-                $is_date   = 0;
-                $is_printf = 0;
-            }
-            redo;
-            };
-        ($state == BEG || $state == BEGM)
-            && m/^([\s\t\n]*)/gcx
-            && do { redo; };
-        # begin ()
-        $state == BEG && m/^([\S\(]) /gcx && do {
-            $state = (($1 eq '(') ? PAR : NUL);
-            redo;
-        };
-        $state == BEGM && m/^([\(])  /gcx && do { $state = PARM; redo };
-
-        # begin or end of string
-        $state == PAR && m/^\s*(\')  /gcx && do { $state = QUO1; redo; };
-        $state == QUO1 && m/^([^\']+)/gcx && do { $str .= $1; redo; };
-        $state == QUO1 && m/^\'  /gcx && do { $state = PAR; redo; };
-
-        $state == PAR && m/^\s*\"  /gcx && do { $state = QUO2; redo; };
-        $state == QUO2 && m/^([^\"]+)/gcx && do { $str .= $1; redo; };
-        $state == QUO2 && m/^\"  /gcx && do { $state = PAR; redo; };
-
-        $state == PAR && m/^\s*\`  /gcx && do { $state = QUO3; redo; };
-        $state == QUO3 && m/^([^\`]*)/gcx && do { $str .= $1; redo; };
-        $state == QUO3 && m/^\`  /gcx && do { $state = PAR; redo; };
-
-        $state == BEGM && m/^(\') /gcx    && do { $state = QUOM1; redo; };
-        $state == PARM && m/^\s*(\') /gcx && do { $state = QUOM1; redo; };
-        $state == QUOM1 && m/^([^\']+)/gcx && do { $str .= $1; redo; };
-        $state == QUOM1 && m/^\'  /gcx && do { $state = COMM; redo; };
-
-        $state == BEGM && m/^(\") /gcx    && do { $state = QUOM2; redo; };
-        $state == PARM && m/^\s*(\") /gcx && do { $state = QUOM2; redo; };
-        $state == QUOM2 && m/^([^\"]+)/gcx && do { $str .= $1; redo; };
-        $state == QUOM2 && m/^\"  /gcx && do { $state = COMM; redo; };
-
-        $state == BEGM && do { $state = NUL; redo; };
-
-        # end ()
-        (          $state == PAR && m/^\s*[\)]/gcx
-                || $state == PARM && m/^\s*[\)]/gcx
-                || $state == COMM && m/^\s*,/gcx)
-            && do {
-            $state = NUL;
-            $vars =~ s/[\n\r]//g if ($vars);
-            if ($str) {
-                my $expression = {
-                    'expression' => $str,
-                    'filename'   => $filename,
-                    'line'       => $line - (() = $str =~ /\n/g),
-                    'vars'       => $vars
-                };
-                $expression->{'type'} = 'date'   if ($is_date);
-                $expression->{'type'} = 'printf' if ($is_printf);
-
-                &add_expression($expression);
-            }
-            undef $str;
-            undef $vars;
-            redo;
-            };
-
-        # a line of vars
-        $state == PAR  && m/^([^\)]*)/gcx && do { $vars .= $1 . "\n"; redo; };
-        $state == PARM && m/^([^\)]*)/gcx && do { $vars .= $1 . "\n"; redo; };
-    }
-
-    unless ($state == NUL) {
-        my $post = $_;
-        $post =~ s/\A(\s*.*\n.*\n.*)\n(.|\n)+\z/$1\n.../;
-        warn sprintf "Warning: incomplete state just before ---\n%s\n", $post;
+    if ($file =~ m{ / scenari / | [.] task \z | / comment [.] tt2 \z }x) {
+        load_title($file, $_);
     }
 }
 
@@ -385,7 +120,7 @@ foreach my $str (@ordered_strings) {
     my $lexi  = $Lexicon{$ostr} // '';
 
     ## Skip meta information (specific to Sympa)
-    next if ($str =~ /^_\w+\_$/);
+    next if $str =~ /^_\w+\_$/;
 
     $str =~ s/"/\\"/g;
     $lexi =~ s/\\/\\\\/g;
@@ -414,7 +149,8 @@ my $output_file =
 my $out;
 my $pot;
 if (-r $output_file) {
-    open $pot, '+<', $output_file or die "$output_file: $!\n";
+    open $pot, '+<', $output_file
+        or die sprintf "%s: %s\n", $output_file, $ERRNO;
     while (<$pot>) {
         if (1 .. /^$/) { $out .= $_; next }
         last;
@@ -425,7 +161,8 @@ if (-r $output_file) {
     seek $pot, 0, 0;
     truncate $pot, 0;
 } else {
-    open $pot, '>', $output_file or die "$output_file: $!\n";
+    open $pot, '>', $output_file
+        or die sprintf "%s: %s\n", $output_file, $ERRNO;
 }
 select $pot;
 
@@ -500,14 +237,335 @@ foreach my $entry (@ordered_bis) {
 sub add_expression {
     my $param = shift;
 
-    @ordered_strings = (@ordered_strings, $param->{'expression'});
-    push @{$file{$param->{'expression'}}},
-        [$param->{'filename'}, $param->{'line'}, $param->{'vars'}];
-    $type_of_entries{$param->{'expression'}} = $param->{'type'}
-        if ($param->{'type'});
+    push @ordered_strings, $param->{expression};
+    push @{$file{$param->{expression}}},
+        [$param->{filename}, $param->{line}, $param->{vars}];
+    $type_of_entries{$param->{expression}} = $param->{type}
+        if $param->{type};
 
 }
 
+sub load_tt2 {
+    my $filename = shift;
+    my $t        = shift;
+    my $filters  = shift;
+
+    # Initiliazing filter names with defaults if necessary.
+    # Defaults stored separately because GetOptions append arguments to
+    # defaults.
+    # Building the string to insert into the regexp that will search strings
+    # to extract.
+    my $tt2_filters = join('|', @{$filters || []}) || 'locdt|loc';
+
+    my ($tag_s, $tag_e);
+    if ($filename eq 'default/mhonarc-ressources.tt2') {
+        # Template Toolkit with ($tag$%...%$tag$) in mhonarc-ressources.tt2
+        # (<=6.2.60; OBSOLETED)
+        ($tag_s, $tag_e) = (qr{[(]\$tag\$%}, qr{%\$tag\$[)]});
+    } elsif ($filename eq 'default/mhonarc_rc.tt2') {
+        # Template Toolkit with <%...%> in mhonarc_rc.tt2 (6.2.61b.1 or later)
+        ($tag_s, $tag_e) = (qr{<%}, qr{%>});
+    } elsif ($filename =~ /[.]tt2\z/) {
+        # Template Toolkit with [%...%]
+        ($tag_s, $tag_e) = (qr{[[]%}, qr{%[]]});
+    } else {
+        die 'bug in logic. Ask developer';
+    }
+
+    my $line;
+
+    $line = 1;
+    pos($t) = 0;
+    while (
+        $t =~ m{
+            \G .*?
+            (?:
+                # Short style: [% "..." | loc(...) %]
+                $tag_s [-=~+]? \s*
+                (?:
+                    \'
+                    ((?: \\. | [^'\\])*)
+                    \'
+                  |
+                    \"
+                    ((?: \\. | [^"\\])*)
+                    \"
+                ) \s*
+                \| \s*
+                ($tt2_filters)
+                (.*?)
+                \s* [-=~+]? $tag_e
+              |
+                # Enclosing style: [%|loc(...)%]...[%END%]
+                $tag_s [-=~+]? \s*
+                \| \s*
+                ($tt2_filters)
+                (.*?)
+                \s* [-=~+]? $tag_e
+                (.*?)
+                $tag_s [-=~+]? \s*
+                END
+                \s* [-=~+]? $tag_e
+            )
+        }gsx
+    ) {
+        my $is_short = $3;
+        my ($this_tag, $vars, $str) =
+            $is_short ? ($3, $4, $1 // $2) : ($5, $6, $7);
+        $line += (() = ($MATCH =~ /\n/g));    # cryptocontext!
+        if ($is_short) {
+            $str =~ s{\\(.)}{
+                ($1 eq 't') ? "\t" :
+                ($1 eq 'n') ? "\n" :
+                ($1 eq 'r') ? "\r" :
+                $1
+            }eg;
+            $vars =~ s/^\s*[(](.*?)[)].*/$1/ or $vars = '';
+        } else {
+            $str =~ s/\\\'/\'/g;
+            $vars =~ s/^\s*\(//;
+            $vars =~ s/\)\s*$//;
+        }
+
+        add_expression(
+            {   expression => $str,
+                filename   => $filename,
+                line       => $line,
+                vars       => $vars,
+                (($this_tag eq 'locdt') ? (type => 'date') : ())
+            }
+        );
+    }
+}
+
+sub load_perl {
+    my $filename = shift;
+    my $t        = shift;
+
+    my $line;
+
+    $t =~ s{(?<=\n)__END__\n.*}{}s;    # Omit postamble
+
+    # Sympa variables (gettext_comment, gettext_id and gettext_unit)
+    $line = 1;
+    pos($t) = 0;
+    while (
+        $t =~ m{
+            \G .*?
+            ([\"\']?)
+            (gettext_comment | gettext_id | gettext_unit)
+            \1
+            \s* => \s*
+            (?:
+                (\") ((?: \\. | [^\"])+) \"
+              | (\') ((?: \\. | [^\'])+) \'
+            )
+        }gsx
+    ) {
+        my ($quot, $str) = ($3 // $5, $4 // $6);
+        $line += (() = ($MATCH =~ /\n/g));    # cryptocontext!
+        $str =~ s{(\\.)}{eval "$quot$1$quot"}esg;
+
+        add_expression(
+            {   expression => $str,
+                filename   => $filename,
+                line       => $line
+            }
+        );
+    }
+
+    # Perl source file
+    my $state = 0;
+    my $str;
+    my $vars;
+    my $type;
+
+    pos($t) = 0;
+    my $orig = 1 + (() = ((my $tmp = $t) =~ /\n/g));
+PARSER: {
+        $t = substr $t, pos $t if pos $t;
+        my $line = $orig - (() = ((my $tmp = $t) =~ /\n/g));
+        # maketext or loc or _
+        if (    $state == NUL
+            and $t =~ m/\b(
+                translate
+              | gettext(?:_strftime|_sprintf)?
+              | maketext
+              | __?
+              | loc
+              | x
+            )/cgx
+        ) {
+            if ($1 eq 'gettext_strftime') {
+                $state = BEGM;
+                $type  = 'date';
+            } elsif ($1 eq 'gettext_sprintf') {
+                $state = BEGM;
+                $type  = 'printf';
+            } else {
+                $state = BEG;
+                undef $type;
+            }
+            redo;
+        }
+        if (($state == BEG or $state == BEGM) and $t =~ m/^([\s\t\n]*)/cg) {
+            redo;
+        }
+        # begin ()
+        if ($state == BEG and $t =~ m/^([\S\(])/cg) {
+            $state = ($1 eq '(') ? PAR : NUL;
+            redo;
+        }
+        if ($state == BEGM and $t =~ m/^([\(])/cg) {
+            $state = PARM;
+            redo;
+        }
+
+        # begin or end of string
+        if ($state == PAR and $t =~ m/^\s*(\')/cg) {
+            $state = QUO1;
+            redo;
+        }
+        if ($state == QUO1 and $t =~ m/^([^\']+)/cg) {
+            $str .= $1;
+            redo;
+        }
+        if ($state == QUO1 and $t =~ m/^\'/cg) {
+            $state = PAR;
+            redo;
+        }
+
+        if ($state == PAR and $t =~ m/^\s*\"/cg) {
+            $state = QUO2;
+            redo;
+        }
+        if ($state == QUO2 and $t =~ m/^([^\"]+)/cg) {
+            $str .= $1;
+            redo;
+        }
+        if ($state == QUO2 and $t =~ m/^\"/cg) {
+            $state = PAR;
+            redo;
+        }
+
+        if ($state == PAR and $t =~ m/^\s*\`/cg) {
+            $state = QUO3;
+            redo;
+        }
+        if ($state == QUO3 and $t =~ m/^([^\`]*)/cg) {
+            $str .= $1;
+            redo;
+        }
+        if ($state == QUO3 and $t =~ m/^\`/cg) {
+            $state = PAR;
+            redo;
+        }
+
+        if ($state == BEGM and $t =~ m/^(\')/cg) {
+            $state = QUOM1;
+            redo;
+        }
+        if ($state == PARM and $t =~ m/^\s*(\')/cg) {
+            $state = QUOM1;
+            redo;
+        }
+        if ($state == QUOM1 and $t =~ m/^([^\']+)/cg) {
+            $str .= $1;
+            redo;
+        }
+        if ($state == QUOM1 and $t =~ m/^\'/cg) {
+            $state = COMM;
+            redo;
+        }
+
+        if ($state == BEGM and $t =~ m/^(\")/cg) {
+            $state = QUOM2;
+            redo;
+        }
+        if ($state == PARM and $t =~ m/^\s*(\")/cg) {
+            $state = QUOM2;
+            redo;
+        }
+        if ($state == QUOM2 and $t =~ m/^([^\"]+)/cg) {
+            $str .= $1;
+            redo;
+        }
+        if ($state == QUOM2 and $t =~ m/^\"/cg) {
+            $state = COMM;
+            redo;
+        }
+
+        if ($state == BEGM) {
+            $state = NUL;
+            redo;
+        }
+
+        # end ()
+        if (   ($state == PAR and $t =~ m/^\s*[\)]/cg)
+            or ($state == PARM and $t =~ m/^\s*[\)]/cg)
+            or ($state == COMM and $t =~ m/^\s*,/cg)) {
+            $state = NUL;
+            $vars =~ s/[\n\r]//g if $vars;
+
+            add_expression(
+                {   expression => $str,
+                    filename   => $filename,
+                    line       => $line - (() = $str =~ /\n/g),
+                    vars       => $vars,
+                    ($type ? (type => $type) : ())
+                }
+            ) if $str;
+            undef $str;
+            undef $vars;
+            redo;
+        }
+
+        # a line of vars
+        if ($state == PAR and $t =~ m/^([^\)]*)/cg) {
+            $vars .= $1 . "\n";
+            redo;
+        }
+        if ($state == PARM and $t =~ m/^([^\)]*)/cg) {
+            $vars .= $1 . "\n";
+            redo;
+        }
+    }
+
+    unless ($state == NUL) {
+        my $post = $t;
+        $post =~ s/\A(\s*.*\n.*\n.*)\n(.|\n)+\z/$1\n.../;
+        warn sprintf "Warning: incomplete state just before ---\n%s\n", $post;
+    }
+}
+
+sub load_title {
+    my $filename = shift;
+    my $t        = shift;
+
+    my $line;
+
+    # Titles in scenarios, tasks and comment.tt2 (title.gettext)
+    $line = 1;
+    pos($t) = 0;
+    while (
+        $t =~ m{
+            \G .*?
+            title [.] gettext \s*
+            ([^\n]+)
+        }gsx
+    ) {
+        my $str = $1;
+        $line += (() = ($MATCH =~ /\n/g));    # cryptocontext!
+
+        add_expression(
+            {   expression => $str,
+                filename   => $filename,
+                line       => $line
+            }
+        );
+    }
+}
+
 sub output {
     my $str = shift // '';
 
@@ -523,74 +581,29 @@ sub output {
 
         ## Move empty lines to previous line as \n
         my $current_line;
-        foreach my $i (0 .. $#lines) {
-            if ($lines[$i] eq '') {
-                if ($#output_lines < 0) {
+        foreach my $line (@lines) {
+            if ($line eq '') {
+                unless (@output_lines) {
                     $current_line .= '\n';
                     next;
                 } else {
-                    $output_lines[$#output_lines] .= '\n';
+                    $output_lines[-1] .= '\n';
                     next;
                 }
             } else {
-                $current_line .= $lines[$i];
+                $current_line .= $line;
             }
             push @output_lines, $current_line;
             $current_line = '';
         }
 
-        ## Add \n unless
-        foreach my $i (0 .. $#output_lines) {
-            if ($i == $#output_lines) {
-                ## No additional \n
-                print "\"$output_lines[$i]\"\n";
-            } else {
-                print "\"$output_lines[$i]\\n\"\n";
-            }
-        }
-
+        # Add \n unless the last line
+        print "\"" . join("\\n\"\n\"", @output_lines) . "\"\n";
     } else {
         print "\"$str\"\n";
     }
 }
 
-sub escape {
-    my $text = shift;
-    $text =~ s/\b_(\d+)/%$1/;
-    return $text;
-}
-
-## Dump a variable's content
-sub dump_var {
-    my ($var, $level, $fd) = @_;
-
-    return undef unless ($fd);
-
-    if (ref($var)) {
-        if (ref($var) eq 'ARRAY') {
-            foreach my $index (0 .. $#{$var}) {
-                print $fd "\t" x $level . $index . "\n";
-                &dump_var($var->[$index], $level + 1, $fd);
-            }
-        } elsif (ref($var) eq 'HASH'
-            || ref($var) eq 'Scenario'
-            || ref($var) eq 'List') {
-            foreach my $key (sort keys %{$var}) {
-                print $fd "\t" x $level . '_' . $key . '_' . "\n";
-                &dump_var($var->{$key}, $level + 1, $fd);
-            }
-        } else {
-            printf $fd "\t" x $level . "'%s'" . "\n", ref($var);
-        }
-    } else {
-        if (defined $var) {
-            print $fd "\t" x $level . "'$var'" . "\n";
-        } else {
-            print $fd "\t" x $level . "UNDEF\n";
-        }
-    }
-}
-
 package CPANFile;
 
 use strict;
